<?php

namespace progress\models;
use progress\models\pdo\MysqlPDO as MysqlPDO;

/**
 * Description of Product
 *
 * @author Artem Yuriev art3mk4@gmail.com Feb 15, 2019 11:02:15 AM
 */
class Product extends MysqlPDO
{

    /**
     * productValue
     * 
     * @var type 
     */
    protected $productValue = array();

    /**
     * productIds
     * 
     * @var type 
     */
    protected $productIds = array();

    /**
     * xmlGenerator
     * 
     * @var type 
     */
    public $xmlGenerator;

    /**
     * images
     * 
     * @var type 
     */
    public $images = array();

    /**
     *
     * @var type 
     */
    protected $sizeProducts = array();

    /**
     *
     * @var type 
     */
    protected $typeProducts = array();

    /**
     *
     * @var type 
     */
    protected $colorProducts = array();

    /**
     *
     * @var type 
     */
    protected $modelProducts = array();

    /**
     *
     * @var type 
     */
    protected $variationProducts = array();

    /**
     *
     * @var type 
     */
    protected $variationsPrices = array();

    /**
     *
     * @var type 
     */
    protected $materialProducts = array();

    /**
     * Product constructor
     */
    public function __construct()
    {
        $this->xmlGenerator = new XmlGenerator();
        $this->setTable('ads_products');
        MysqlPDO::__construct();
    }

    /**
     * getFields
     * 
     * @return type
     */
    public function getFields()
    {
        return array(
            'id', 'image_link', 'price', 'title', 'google_product_category',
            'description', 'brand', 'availability', 'link', 'condition'
        );
    }

    /*
     * getTotalProducts
     */
    public function getTotalProducts($category_slug = '')
    {
        $selectSql = 'SELECT count(DISTINCT p.id) as count FROM ' . TABLE_PREFIX . $this->table . ' p ' .
            'INNER JOIN ' . TABLE_PREFIX . 'posts posts ON p.post_id = posts.ID ' .
            'LEFT JOIN ' . TABLE_PREFIX . 'term_relationships tr ON (posts.ID = tr.object_id) ' .
            'LEFT JOIN ' . TABLE_PREFIX . 'term_taxonomy tt ON (tr.term_taxonomy_id = tt.term_taxonomy_id) ' .
            'LEFT JOIN ' . TABLE_PREFIX .'terms t ON (tt.term_id = t.term_id) ' .
            'WHERE posts.post_status = \'publish\'';

        if (is_array($category_slug)) {
            $categoryString = implode("', '", $category_slug);
        } else {
            $categoryString = $category_slug;
        }

        if (!empty($categoryString)) {
            $selectSql .= " AND tt.taxonomy = 'product_cat'
                AND t.slug in ('{$categoryString}')";
        } else {
            $selectSql .= " AND tt.taxonomy = 'product_cat' AND tt.parent = 0";
        }

        $select = $this->prepare($selectSql);
        $select->execute();
        return $select->fetch(\PDO::FETCH_ASSOC);
    }

    /**
     * findAll
     * 
     * @param type $page
     * @param type $category_slug
     * @param type $currency
     */
    public function findAll($page = 1, $category_slug = '', $currency = '')
    {
        $limit = 1;
        $start = ($page - 1) * $limit;
        $sql = "SELECT product.id as id,
            post.ID as post_id,
            product.imageUrl as image_link,
            product.price as price,
            product.priceMax as priceMax,
            product.salePrice as salePrice,
            product.salePriceMax as salePriceMax,
            product.currency as currency,
            product.discount as discount,
            post.post_title as title,
            t.name as google_product_category,
            relation.google_category_text as alias_category,
            (SELECT MAX(identifier_exists)
                FROM " . TABLE_PREFIX . "ads_googlemerchant_category_relation relation
                WHERE relation.term_id = t.term_id
            ) identifier_exists,
            t.term_id as category_id,
            post.post_content as description,
            MAX(CASE WHEN attributes.attr_name = 'Gender' then attributes.attr_value ELSE NULL END) as gender,
            MAX(CASE WHEN attributes.attr_name = 'Age Group' then attributes.attr_value ELSE NULL END) as age_group,
            MAX(CASE WHEN attributes.attr_name = 'Department Name' then attributes.attr_value ELSE NULL END) as department_name,
            meta.gallery as gallery,
            meta.sku as sku,
            meta.skuAttr as skuAttr,
            alimeta.product_id as mpn
            FROM " . TABLE_PREFIX . $this->table . " product
            INNER JOIN " . TABLE_PREFIX . "posts post ON product.post_id = post.ID
            LEFT JOIN " . TABLE_PREFIX . "ads_products_meta meta ON (meta.post_id = post.ID)
            LEFT JOIN " . TABLE_PREFIX . "ads_ali_meta alimeta ON (alimeta.post_id = post.ID)
            LEFT JOIN " . TABLE_PREFIX . "ads_attributes attributes ON (attributes.post_id = post.ID)
            LEFT JOIN " . TABLE_PREFIX . "term_relationships tr ON (post.ID = tr.object_id)
            LEFT JOIN " . TABLE_PREFIX . "term_taxonomy tt ON (tr.term_taxonomy_id = tt.term_taxonomy_id)
            LEFT JOIN " . TABLE_PREFIX . "terms t ON (tt.term_id = t.term_id)
            LEFT JOIN " . TABLE_PREFIX . "ads_googlemerchant_category_relation relation ON relation.term_id = t.term_id
            WHERE post.post_type='product'
            AND post.post_status = 'publish'";

        if (is_array($category_slug)) {
            $categoryString = implode("', '", $category_slug);
        } else {
            $categoryString = $category_slug;
        }

        if (!empty($categoryString)) {
            $sql .= " AND tt.taxonomy = 'product_cat'
                AND t.slug in ('{$categoryString}')";
            $parent = true;
        } else {
            $sql .= " AND tt.taxonomy = 'product_cat' AND tt.parent = 0";
            $parent = false;
        }

        $sql .= " GROUP BY post.ID, post.post_title LIMIT :start, :limit";
        $select = $this->prepare($sql);
        $select->bindValue(':start', $start, \PDO::PARAM_INT);
        $select->bindValue(':limit', $limit, \PDO::PARAM_INT);
        $select->execute();
        $result = $select->fetchAll(\PDO::FETCH_ASSOC);
        foreach ($result as $value) {
            $this->images = array();
            $params = $this->parseSku(
                Settings::maybe_unserialize($value['sku']),
                Settings::maybe_unserialize($value['skuAttr'])
            );
            $value['image_link'] = $this->getImageLink($value);
            unset($value['gallery']);
            $value['id'] = $value['post_id'];
            $value['availability'] = 'in stock';
            $value['link'] = $this->getLink($value['post_id']);
            $value['price'] = $this->getPriceText($value, $currency);
            $value['select_currency'] = $currency;
            $value['condition'] = 'new';
            $value['google_product_category'] = $this->getGoogleProductCategory($value, $parent);
            $value['description'] = $this->getDescription($value);
            $value['brand'] = WP_ADS_BLOGNAME;
            $value['title'] = $this->getTitle($value['title']);
            $value['age_group'] = $this->getAgeGroup($value);
            $value['gender'] = $this->getGender($value);
            $value['itemId'] = $value['id'];
            if (!isset($value['identifier_exists'])) {
                $value['identifier_exists'] = 0;
            }
            $this->multiplyProduct($value, $params);
        }
    }

    /**
     * getTitle
     * 
     * @param type $title
     * @return type
     */
    public function getTitle($title)
    {
        if (strlen($title) > 150) {
            return mb_substr($title, 0, 146) . '...';
        }

        return $title;
    }

    /**
     * getLink
     * 
     * @param type $postId
     * @return string
     */
    public function getLink($postId)
    {
        $post = new Post();
        $product['link'] = $post->get_post_permalink($postId);
        if (empty($product['link'])) {
            $product['link'] = '#';
        }

        return $product['link'];
    }

    /**
     * getDescription
     * 
     * @param type $product
     * @return type
     */
    public function getDescription($product)
    {
        if (!isset($product['description']) || empty($product['description'])) {
            return XmlGenerator::getEmptyDescription();
        }

        return XmlGenerator::removeTableChart($product['description']);
    }

    /**
     * convertPrice
     * 
     * @param type $price
     * @return type
     */
    public function convertPrice($price)
    {
        return $this->roundsCentPrice($price);
    }

    /**
     * getPriceText
     * 
     * @param type $params
     * @param type $currency
     * @return type
     */
    public function getPriceText($params, $currency)
    {
        $priceText  = '';
        $priceText .= $this->checkSalePrice($params, $currency);
        $priceText .= $this->checkPriceMax($params, $currency);
        $priceText .= $this->checkPrice($params, $currency);
        if ($priceText == '') {
            return $this->getPrice($params['salePrice'], $params['currency'], $currency);
        }

        return $priceText;
    }

    /**
     * checkSalePrice
     * 
     * @param type $params
     * @param type $default
     * @return string
     */
    public function checkSalePrice($params, $default)
    {
        if ($params['price'] != $params['salePrice']
            && $params['priceMax'] == 0
        ) {
            return $this->getPrice($params['salePrice'], $params['currency'], $default);
        }

        return '';
    }

    /**
     * checkPriceMax
     * 
     * @param type $params
     * @param type $default
     * @return string
     */
    public function checkPriceMax($params, $default)
    {
        if ($params['price'] == $params['salePrice']
            && $params['priceMax'] != 0
            && $params['priceMax'] != $params['price']
        ) {
            return $this->getPrice($params['price'], $params['currency'], $default);
        }

        return '';
    }

    /**
     * checkPrice
     * 
     * @param type $params
     * @param type $default
     * @return string
     */
    public function checkPrice($params, $default)
    {
        $discount = $this->checkDiscount($params, false);
        if ($params['priceMax'] != 0
            && $params['priceMax'] != $params['price']
            && $discount != ''
        ) {
            return $this->getPrice(
                $params['salePrice'],
                $params['currency'],
                $default
            );
        }

        return '';
    }

    /**
     * getPrice
     * 
     * @param type $price
     * @param type $fromCurrency
     * @param type $toCurrency
     * @return string
     */
    public function getPrice($price, $fromCurrency, $toCurrency)
    {
        $priceCurrency = $this->ads_get_currency_symbol($toCurrency);
        $priceConverted = $this->ads_price_convert_currents(
            $price,
            $fromCurrency,
            $toCurrency
        );

        if ($priceCurrency['pos'] == 'before') {
            $priceText = $toCurrency . ' ' . $this->convertPrice($priceConverted);
        } else {
            $priceText = $this->convertPrice($priceConverted) . ' ' . $toCurrency;
        }

        return $priceText;
    }

    /**
     * checkDiscount
     * 
     * @param type $params
     * @param type $compare
     * @return string
     */
    public function checkDiscount($params, $compare = false)
    {
        if ($params['price'] > 0 && $compare) {
            $discount = round((($params['price'] - $params['salePrice']) / $params['price']) * 100);
            return ($discount > 0) ? $discount : '';
        }

        if ($params['discount'] > 0) {
            return $params['discount'];
        }

        return '';
    }

    /**
     * addSizeProduct
     * 
     * @param type $index
     * @param type $product
     */
    public function addSizeProduct($index, $product)
    {
        $this->sizeProducts[$index] = $product;
    }

    /**
     * getSizeProducts
     * 
     * @return type
     */
    public function getSizeProducts()
    {
        return $this->sizeProducts;
    }

    /**
     * addModelProduct
     * 
     * @param type $product
     */
    public function addModelProduct($index, $product)
    {
        $this->modelProducts[$index] = $product;
    }

    /**
     * getModelProducts
     * 
     * @return type
     */
    public function getModelProducts()
    {
        return $this->modelProducts;
    }

    /**
     * addVariationProduct
     * 
     * @param type $product
     */
    public function addVariationProduct($index, $product)
    {
        $this->variationProducts[$index] = $product;
    }

    /**
     * getVariations
     * 
     * @return type
     */
    public function getVariationProducts()
    {
        return $this->variationProducts;
    }

    /**
     * addMaterialProduct
     * 
     * @param type $index
     * @param type $product
     */
    public function addMaterialProduct($index, $product)
    {
        $this->materialProducts[$index] = $product;
    }

    /**
     * getMaterialProducts
     * 
     * @return type
     */
    public function getMaterialProducts()
    {
        return $this->materialProducts;
    }

    /**
     * addTypeProduct
     * 
     * @param type $product
     */
    public function addTypeProduct($index, $product)
    {
        $this->typeProducts[$index] = $product;
    }

    /**
     * getTypeProducts
     * 
     * @return type
     */
    public function getTypeProducts()
    {
        return $this->typeProducts;
    }

    /**
     * setVariationsPrices
     * 
     * @param type $prices
     */
    public function setVariationsPrices($attributes)
    {
        if (isset($attributes['price'])) {
            $this->variationsPrices = $attributes['price'];
        }
    }

    /**
     * getVariationsPrices
     * 
     * @return type
     */
    public function getVariationsPrices()
    {
        return $this->variationsPrices;
    }

    /**
     * addImage
     * 
     * @param type $value
     * @param type $field
     */
    public function addImage($value, $field)
    {
        if (isset($value['img'])
            && !empty($value['img'])
        ) {
            $this->images[$field] = $this->getGalleryImage($value['img']);
        }
    }

    /**
     * addImages
     * 
     * @param type $images
     */
    public function addImages($images, $value)
    {
        if (isset($images['images'])) {
            $tempImages = array();
            foreach ($images['images'] as $key => $item) {
                $tempImages[$key] = $item;
                if ($item == false
                    && isset($value['image_link'])
                ) {
                    $tempImages[$key] = $value['image_link'];
                }
            }
            $this->images = $tempImages;
        }
    }

    /**
     * 
     * @return type
     */
    public function getImages()
    {
        return $this->images;
    }

    /**
     * getImage
     * 
     * @param type $color
     */
    public function getImage($color)
    {
        $images = $this->getImages();
        if (isset($images[$color])) {
            return $images[$color];
        }

        return false;
    }

    /**
     * addColorProduct
     * 
     * @param type $product
     */
    public function addColorProduct($index, $product)
    {
        $this->colorProducts[$index] = $product;
    }

    /**
     * getColorProducts
     */
    public function getColorProducts()
    {
        return $this->colorProducts;
    }

    /**
     * getGoogleProductCategory
     * 
     * @param string $product
     * @return string
     */
    public function getGoogleProductCategory($product)
    {
        if (isset($product['alias_category'])) {
            return $product['alias_category'];
        }

        if (isset($product['category_id']) && !empty($product['category_id'])) {
            return $this->getListCategory($product);
        }

        return ' ';
    }

    /**
     * getListCategory
     * 
     * @param type $product
     */
    public function getListCategory($product)
    {
        $categories = array();
        $select = $this->prepare('SELECT t.name FROM ' . TABLE_PREFIX . 'terms AS t
            INNER JOIN ' . TABLE_PREFIX . 'term_taxonomy AS tt ON (tt.term_id = t.term_id)
            INNER JOIN ' . TABLE_PREFIX . 'term_relationships AS tr ON (tr.term_taxonomy_id = tt.term_taxonomy_id)
            WHERE tr.object_id = :post_id AND tt.taxonomy = \'product_cat\' ORDER BY t.name ASC');
        $select->bindValue(':post_id', $product['post_id']);
        $select->execute();
        $result = $select->fetchAll(\PDO::FETCH_ASSOC);
        if (!$result) {
            return ' ';
        }

        $categories = array_map(
            function($value) {
                return $value['name'];
            },
            $result
        );

        return implode(' > ', $categories);
    }

    /**
     * parseSku
     * 
     * @param type $skuParams
     * @param type $skuAttr
     * @return array
     */
    public function parseSku($skuParams, $skuAttr)
    {
        $nameParams = array(
            'colorNames'     => ShopConstant::colorNames(),
            'sizeNames'      => ShopConstant::sizeNames(),
            'typeNames'      => ShopConstant::typeNames(),
            'modelNames'     => ShopConstant::modelNames(),
            'variationNames' => ShopConstant::variationNames(),
            'materialNames'  => ShopConstant::materialNames()
        );

        $result = array();
        if (!is_array($skuParams)) {
            return $result;
        }

        $prices = array();
        foreach ($skuAttr as $key => $attribute) {
            $tempPrice = !empty((float)$attribute['salePrice']) ? $attribute['salePrice'] : $attribute['price'];
            $prices[$key] = $tempPrice;
        }

        foreach ($prices as $keys => $value) {
            $keyPieces = explode(';', $keys);
            foreach ($keyPieces as $key) {
                if (!isset($skuParams[$key])
                    || !isset($skuParams[$key]['prop_title'])
                    || !isset($skuParams[$key]['title'])
                ) {
                    continue;
                }

                $keyTitle = $skuParams[$key]['prop_title'];
                $keyValue = $skuParams[$key]['title'];
                foreach ($nameParams as $name  => $params) {
                    if (!isset($result[$keyTitle][$keys])
                        && in_array($keyTitle, $params)
                    ) {
                        $result[$keyTitle][$keys] = $keyValue;
                        if (!isset($result['price'][$keys])) {
                            $this->setImages($skuParams[$key], $keys);
                            $result['price'][$keys] = $value;
                        }
                    }
                }
            }
        }

        return $result;
    }

    /**
     * setImages
     * 
     * @param type $value
     * @param type $field
     */
    public function setImages($value, $field)
    {
        if (isset($value['img'])
            && !empty($value['img'])
        ) {
            $this->images[$field] = $this->getGalleryImage($value['img']);
        }
    }

    /**
     * roundsCentPrice
     * 
     * @param type $price
     * @return type
     */
    public function roundsCentPrice($price)
    {
        if ($price == 0) {
            return $price;
        }

	if (ADS_PRICE_ROUNDING == 1) {
            $price = round($price);
	}

	$assignCents = defined('ADS_PRICE_ASSIGNCENTS') && !empty(ADS_PRICE_ASSIGNCENTS) ? explode(',', ADS_PRICE_ASSIGNCENTS) : false;
	$price   = $this->ads_floatvalue($price);
	$priceAR = explode('.', $price);

	if (intval($priceAR[0]) == 0) {
            if ($assignCents && ($key = array_search('00', $assignCents)) !== false) {
                unset($assignCents[$key]);
            }
	}

	if ($assignCents) {
            $priceAR[1] = isset($priceAR[1] ) ? $priceAR[1] : 0;
            $lastPriceKey = $this->searchNearest( $priceAR[1], $assignCents);
            $priceAR[1] = $assignCents[$lastPriceKey];
	}

	$priceAR[1] = isset($priceAR[1]) ? $priceAR[1] : '';
	$price = $priceAR[1] != '' ? $priceAR[0] . '.' . $priceAR[1] : $priceAR[0];
	return $price;
    }

    /**
     * searchNearest
     * 
     * @param type $value
     * @param type $inArray
     * @return type
     */
    public function searchNearest($value, $inArray)
    {
        $lastKey = null;
        $lastDif = null;

        foreach ($inArray as $k => $v) {
            if ($v == $value) {
                return $k;
            }

            $dif = abs($value - $v);
            if (is_null($lastKey)
                || $dif < $lastDif
            ) {
                $lastKey = $k;
                $lastDif = $dif;
            }
        }

        return $lastKey;
    }

    /**
     * ads_get_currency_symbol
     * 
     * @param type $cur
     * @return type
     */
    public function ads_get_currency_symbol($cur)
    {
	$foo = Helper::ads_get_list_currency();
	return isset($foo[$cur]) ? $foo[$cur] : $cur;
    }

    /**
     * ads_price_convert_currents
     * 
     * @param type $price
     * @param type $from
     * @param type $to
     */
    public function ads_price_convert_currents($price, $from, $to = 'USD')
    {
	$price     = $this->ads_floatvalue($price);
	$curValues = @unserialize(ADS_CUVALUE);

	if ($from == $to) {
            return $price;
        }

	if (!isset($curValues[$from])
            || !isset($curValues[$to])
        ) {
            return $price;
        }

	$sub   = $curValues[$to]/$curValues[$from];
	$price = $price * $sub;
	return round($price, 2);
    }

    /**
     * ads_floatvalue
     * 
     * @param type $value
     * @return type
     */
    public function ads_floatvalue($value)
    {
	$value = html_entity_decode($value, ENT_QUOTES, "UTF-8");
	$value = preg_replace('/[^0-9,.]/', '', $value, -1 );

	if (preg_match('/(\d+\,\d+)+\.\d+/', $value)) {
            $value = str_replace(',', '', $value);
	}

        $value = str_replace(',', '.', $value);
        return number_format(floatval($value), 2, '.', '');
    }

    /**
     * get_attached_file
     * 
     * @param type $post_id
     * @return type
     */
    public function get_attached_file($post_id)
    {
        $select = $this->prepare('select * from ' . TABLE_PREFIX . 'postmeta where meta_key = \'_wp_attached_file\' and post_id = :post_id');
        $select->bindValue(':post_id', $post_id);
        $select->execute();
        $result = $select->fetch(\PDO::FETCH_ASSOC);
        if (!is_array($result)
            || !isset($result['meta_value'])
        ) {
            return $result;
        }

        $filename = ABSPATH . DIRECTORY_SEPARATOR . 'uploads' . DIRECTORY_SEPARATOR . $result['meta_value'];
        if (!file_exists($filename)) {
            return false;
        }

        return $result;
    }

    /**
     * wp_get_attachment_url
     * 
     * @param type $filePath
     */
    public function wp_get_attachment_url($filePath)
    {
        return ADS_FACEBOOK_SITEURL . '/wp-content/uploads/' .  $filePath['meta_value'];
    }

    /**
     * getImageLink
     * 
     * @param type $value
     */
    public function getImageLink($value)
    {
        $gallery = Settings::maybe_unserialize($value['gallery']);
        if (is_array($gallery)) {
            foreach ($gallery as $item) {
                $url = $this->getGalleryImage($item);
                if (!empty($url)) {
                    return $url;
                }
            }
        }

        $filePath = $this->get_attached_file($value['id']);
        if (!empty($filePath)) {
            return $this->wp_get_attachment_url($value['id']);
        }

        return $this->getResizeImage($value['image_link'], 'ads-large');
    }

    /**
     * getAgeGroup
     * 
     * @param type $value
     * @return string
     */
    public function getAgeGroup($value)
    {
        if (isset($value['adult'])) {
            return ShopConstant::parseAgeGroup($value['adult']);
        }

        if (isset($value['department_name'])) {
            return ShopConstant::parseAgeGroup($value['department_name']);
        }

        return 'adult';
    }

    /**
     * getGender
     * 
     * @param type $value
     * @return string
     */
    public function getGender($value)
    {
        if (!isset($value['gender'])) {
            return 'unisex';
        }

        return ShopConstant::parseGender($value['gender']);
    }

    /**
     * add
     * 
     * @param type $product
     */
    public function add($product)
    {
        XmlGenerator::addItem($product);
    }

    /**
     * multiplyProduct
     * 
     * @param type $value
     * @param type $attributes
     */
    public function multiplyProduct($value, $attributes)
    {
        $this->resetProducts();
        if (empty($attributes)) {
            $this->productValue['size'] = '';
            $this->productValue['color'] = '';
            $this->productValue['type'] = '';
            $this->productValue['model'] = '';
            $this->productValue['variation'] = '';
            $this->productValue['material'] = '';
            $this->addProduct($value);
        } else {
            $this->addColor($attributes);
            $this->addType($attributes);
            $this->addSize($attributes);
            $this->addModel($attributes);
            $this->addVariation($attributes);
            $this->addMaterial($attributes);
            $this->addImages($attributes, $value);
            $this->setVariationsPrices($attributes);
            $this->addProduct($value);
            $this->addProducts('color', $this->getColorProducts());
            $this->addProducts('type', $this->getTypeProducts());
            $this->addProducts('size', $this->getSizeProducts());
            $this->addProducts('model', $this->getModelProducts());
            $this->addProducts('variation', $this->getVariationProducts());
            $this->addProducts('material', $this->getMaterialProducts());
        }
    }

    /**
     * addProduct
     * 
     * @param type $product
     */
    public function addProduct($product)
    {
        if (!isset($this->productValue['itemId'])
            && empty($this->productValue)
        ) {
            $this->productValue = $product;
            $this->productValue['itemId'] = $product['id'];
        } else {
            $this->productValue = $product;
        }
        
        if (!in_array($this->productValue['itemId'], $this->productIds)) {
            if (count($this->productIds) > 0) {
                $this->add($this->productValue);
            }
            $this->productIds[] = $this->productValue['itemId'];
        }
    }

    /**
     * addProducts
     * 
     * @param type $field
     * @param type $productParams
     */
    public function addProducts($field, $productParams)
    {
        foreach ($productParams as $keyParam => $valueParam) {
            $this->addProductVariation($field, $valueParam, $keyParam);
        }
    }

    /**
     * addProductVariation
     * 
     * @param type $attribute
     * @param type $value
     * @param type $keyParam
     */
    public function addProductVariation($attribute, $value, $keyParam)
    {
        $publishVariation = $this->checkVariation($keyParam);
        if (!$publishVariation) {
            return false;
        }

        $priceStash = $this->getVariationsPrices();
        $imageStash = $this->getImages();
        $colorStash = $this->getColorProducts();
        $materialStash = $this->getMaterialProducts();
        $sizeStash = $this->getSizeProducts();
        $valueProduct = $this->productValue;
        if (!isset($valueProduct[$attribute])
            || $valueProduct[$attribute] != $value
        ) {
            $valueProduct = $this->addVariationSize($valueProduct, $keyParam, $sizeStash);
            $valueProduct = $this->addVariationPrice($valueProduct, $keyParam, $priceStash);
            $valueProduct = $this->addVariationImage($valueProduct, $keyParam, $imageStash);
            $valueProduct = $this->addVariationColor($valueProduct, $keyParam, $colorStash);
            $valueProduct = $this->addVariationMaterial($valueProduct, $keyParam, $materialStash);
            $tempProduct = $valueProduct;
            $tempProduct[$attribute] = $value;
            $tempProduct['itemId'] = $valueProduct['id'] . '-' . $this->getVariationIndex($keyParam);
            $tempProduct['skuLink'] = $valueProduct['link'] . '?sku=' . $keyParam; 
            $this->addProduct($tempProduct);
        }
    }

    /**
     * checkVariation
     * 
     * @param type $keyParam
     * @return boolean
     */
    public function checkVariation($keyParam)
    {
        if (!isset($this->productValue['skuAttr'])) {
            return true;
        }

        $attributes = Settings::maybe_unserialize($this->productValue['skuAttr']);
        if (!is_array($attributes)) {
            return true;
        }

        if (!isset($attributes[$keyParam])) {
            return true;
        }

        if (!isset($attributes[$keyParam]['quantity'])
            || !isset($attributes[$keyParam]['isActivity'])
        ) {
            return true;
        }

        if ($attributes[$keyParam]['quantity'] == 0
            || $attributes[$keyParam]['isActivity'] == 0
        ) {
            return false;
        }

        return true;
    }

    /**
     * getVariationIndex
     * 
     * @param type $variation
     * @return int
     */
    public function getVariationIndex($variation)
    {
        if (!isset($this->productValue['skuAttr'])
            || empty($this->productValue['skuAttr'])
        ) {
            return 0;
        }

        $params = Settings::maybe_unserialize($this->productValue['skuAttr']);
        $keys = array_keys($params);
        $key = array_search($variation, $keys);
        return $key;
   }

    /**
     * addVariationPrice
     * 
     * @param type $value
     * @param type $index
     * @param type $priceStash
     * @return type
     */
    public function addVariationPrice($value, $index, $priceStash)
    {
        if (!isset($value['currency']) && isset($value['select_currency'])) {
            $value['currency'] = $value['select_currency'];
        }

        if (isset($priceStash[$index])) {
            $price = $priceStash[$index];
            $value['price'] = $this->getPrice($price, $value['currency'], $value['select_currency']);
            return $value;
        }

        if (is_array($priceStash)) {
            $price = min($priceStash);
            $value['price'] = $this->getPrice($price, $value['currency'], $value['select_currency']);
            return $value;
        }

        return $value;
    }

    /**
     * addVariationImage
     * 
     * @param type $value
     * @param type $index
     * @param type $imageStash
     * @return type
     */
    public function addVariationImage($value, $index, $imageStash)
    {
        if (isset($imageStash[$index])) {
            $value['image_link'] = $imageStash[$index];
            return $value;
        }

        return $value;
    }

    /**
     * addVariationColor
     * 
     * @param type $value
     * @param type $index
     * @param type $colorStash
     * @return type
     */
    public function addVariationColor($value, $index, $colorStash)
    {
        if (isset($colorStash[$index])) {
            $value['color'] = $colorStash[$index];
            return $value;
        }

        return $value;
    }

    /**
     * addVariationSize
     * 
     * @param type $value
     * @param type $index
     * @param type $sizeStash
     * @return type
     */
    public function addVariationSize($value, $index, $sizeStash)
    {
        if (isset($sizeStash[$index])) {
            $value['size'] = $sizeStash[$index];
            return $value;
        }

        return $value;
    }

    /**
     * addVariationMaterial
     * 
     * @param type $value
     * @param type $index
     * @param type $materialStash
     * @return type
     */
    public function addVariationMaterial($value, $index, $materialStash)
    {
        if (isset($materialStash[$index])) {
            $value['material'] = $materialStash[$index];
            return $value;
        }

        return $value;
    }

    /**
     * resetProducts
     */
    public function resetProducts()
    {
        $this->productIds = array();
        $this->productValue = array();
        $this->sizeProducts = array();
        $this->typeProducts = array();
        $this->colorProducts = array();
        $this->modelProducts = array();
        $this->variationProducts = array();
        $this->variationsPrices = array();
        $this->materialProducts = array();
    }

    /**
     * getGalleryImage
     * 
     * @param type $url
     * @return type
     */
    private function getGalleryImage($url)
    {
        if (!is_numeric($url)) {
            return $url;
        }

        $filePath = $this->get_attached_file($url);
        if (!empty($filePath)) {
            return $this->wp_get_attachment_url($filePath);
        }
    }

    /**
     * getResizeImage
     * 
     * @param type $url
     * @param type $size
     * @return type
     */
    private function getResizeImage($url, $size = 'ads-big')
    {
        $foo = array(
            'ads-thumb'  => '_50x50.jpg',
            'ads-medium' => '_220x220.jpg',
            'ads-big'    => '_350x350.jpg',
            'ads-large'  => '_640x640.jpg'
        );

        if (!isset($foo[$size])) {
            return $url;
        }

        $fullUrl = $this->getFullUrlImg($url);
        return $fullUrl . $foo[ $size ];
    }

    /**
     * getFullUrlImg
     * 
     * @param type $url
     * @return type
     */
    private function getFullUrlImg($url)
    {
        return preg_replace('/_\d+x\d+\.jpg$/', '', $url);
    }

    /**
     * addType
     * 
     * @param type $attributes
     */
    private function addType($attributes)
    {
        $typeNames = ShopConstant::typeNames();
        foreach ($typeNames as $name) {
            if (isset($attributes[$name])) {
                $this->fillTypeProduct($attributes[$name]);
            }
        }
    }

    /**
     * addColor
     * 
     * @param type $attributes
     */
    private function addColor($attributes)
    {
        $colorNames = ShopConstant::colorNames();
        foreach ($colorNames as $name) {
            if (isset($attributes[$name])) {
                $this->fillColorProduct($attributes[$name]);
            }
        }
    }

    /**
     * addSize
     * 
     * @param type $attributes
     */
    private function addSize($attributes)
    {
        $sizeNames = ShopConstant::sizeNames();
        foreach ($sizeNames as $name) {
            if (isset($attributes[$name])) {
                $this->fillSizeProduct($attributes[$name]);
            }
        }
    }

    /**
     * addModel
     * 
     * @param type $attributes
     */
    private function addModel($attributes)
    {
        $modelNames = ShopConstant::modelNames();
        foreach ($modelNames as $name) {
            if (isset($attributes[$name])) {
                $this->fillModelProduct($attributes[$name]);
            }
        }
    }

    /**
     * addVariation
     * 
     * @param type $attributes
     */
    private function addVariation($attributes)
    {
        $variationNames = ShopConstant::variationNames();
        foreach ($variationNames as $name) {
            if (isset($attributes[$name])) {
                $this->fillVariationProduct($attributes[$name]);
            }
        }
    }

    /**
     * addMaterial
     * 
     * @param type $attributes
     */
    private function addMaterial($attributes)
    {
        $materialNames = ShopConstant::materialNames();
        foreach ($materialNames as $name) {
            if (isset($attributes[$name])) {
                $this->fillMaterialProduct($attributes[$name]);
            }
        }
    }

    /**
     * fillSizeProduct
     * 
     * @param type $params
     * @param type $value
     */
    private function fillSizeProduct($params)
    {
        foreach ($params as $index => $item) {
            $this->addSizeProduct($index, $item);
        }
    }

    /**
     * fillColorProduct
     * 
     * @param type $params
     * @param type $value
     * @return string
     */
    private function fillColorProduct($params)
    {
        foreach ($params as $index => $item) {
            $this->addColorProduct($index, $item);
        }
    }

    /**
     * fillTypeProduct
     * 
     * @param type $params
     * @param type $value
     * @return string
     */
    private function fillTypeProduct($params)
    {
        foreach ($params as $index => $item) {
            $this->addTypeProduct($index, $item);
        }
    }

    /**
     * fillModelProduct
     * 
     * @param type $params
     */
    private function fillModelProduct($params)
    {
        foreach ($params as $index => $item) {
            $this->addModelProduct($index, $item);
        }
    }

    /**
     * fillVariationProduct
     * 
     * @param type $params
     */
    private function fillVariationProduct($params)
    {
        foreach ($params as $index => $item) {
            $this->addVariationProduct($index, $item);
        }
    }

    /**
     * fillMaterialProduct
     * 
     * @param type $params
     */
    private function fillMaterialProduct($params)
    {
        foreach ($params as $index => $item) {
            $this->addMaterialProduct($index, $item);
        }
    }
}