<?php

namespace progress\models;

/**
 * Description of XmlGenerator
 *
 * @author Artem Yuriev art3mk4@gmail.com Dec 3, 2018 10:40:32 AM
 */
class XmlGenerator
{

    /**
     * XMLCATALOGFOLDER
     */
    const XMLCATALOGFOLDER = 'tempgooglexml';

    /**
     * XMLCATALOGCOPYFOLDER
     */
    const XMLCATALOGCOPYFOLDER = 'googlexml';

    /**
     * $images
     * 
     * @var type 
     */
    public static $images = array();

    /**
     *
     * @var type 
     */
    public static $doc;

    /**
     * XmlGenerator constructor
     */
    public function __construct()
    {
    }

    /**
     * generateHeader
     * 
     * @return boolean
     */
    public static function generateHeader()
    {
        $folderExists = self::checkFolderExists();
        if (!$folderExists) {
            return false;
        }

        $nsUrl = 'http://base.google.com/ns/1.0';
        self::$doc = new \XMLWriter();
        self::$doc->openUri(self::getFilePath());
        self::$doc->setIndent(true);
        self::$doc->startDocument('1.0', 'utf-8');
        self::$doc->startElement('rss');
        self::$doc->writeAttribute('version', '2.0');
        self::$doc->writeAttribute('xmlns:g', $nsUrl);
        self::$doc->startElement('channel');
        self::$doc->writeElement('title', WP_ADS_BLOGNAME);
        self::$doc->writeElement('description', WP_ADS_BLOGDESCRIPTION);
        self::$doc->writeElement('link', WP_ADS_BLOGURL);
    }

    /**
     * endDocument
     */
    public static function endDocument()
    {
        self::$doc->endElement();
        self::$doc->endElement();
        self::$doc->endElement();
        self::$doc->flush();
    }

    /**
     * checkFolderExists
     * 
     * @param type $file
     * @param type $content
     */
    public static function checkFolderExists()
    {
        $uploadFolder = self::getUploadFolder();
        if (!is_dir($uploadFolder)) {
            mkdir($uploadFolder, 0755);
        }

        $folderDir = self::getFolderDir();
        if (!is_dir($folderDir)) {
            mkdir($folderDir, 0755);
        }

        $copyDir = self::getCopyFolderDir();
        if (!is_dir($copyDir)) {
            mkdir($copyDir, 0755);
        }

        if (is_writable($folderDir)
            && is_writable($copyDir)
        ) {
            return true;
        }

        return false;
    }

    /**
     * checkFileExists
     */
    public static function checkFileExists()
    {
        $file = self::getFilePath();
        if (file_exists($file)) {
            return true;
        }

        return false;
    }

    /**
     * getFilePath
     * 
     * @return type
     */
    public static function getFilePath()
    {
        return self::getFolderDir() . self::getFileName();
    }

    /**
     * getCopyFilePath
     * 
     * @return type
     */
    public static function getCopyFilePath()
    {
        return self::getCopyFolderDir() . self::getFileName();
    }

    /**
     * getFileName
     * 
     * @return string
     */
    public static function getFileName()
    {
        return 'product_catalog.xml';
    }

    /**
     * getFolderDir
     * 
     * @return type
     */
    public static function getFolderDir()
    {
        return self::getUploadFolder() . self::getGoogleXmlFolder();
    }

    /**
     * getCopyFolderDir
     * 
     * @return type
     */
    public static function getCopyFolderDir()
    {
        return self::getUploadFolder() . self::getGoogleXmlCopyFolder();
    }

    /**
     * getGoogleXmlFolder
     * 
     * @return type
     */
    public static function getGoogleXmlFolder()
    {
        return DIRECTORY_SEPARATOR . self::XMLCATALOGFOLDER . DIRECTORY_SEPARATOR;
    }

    /**
     * getGoogleXmlCopyFolder
     * 
     * @return type
     */
    public static function getGoogleXmlCopyFolder()
    {
        return DIRECTORY_SEPARATOR . self::XMLCATALOGCOPYFOLDER . DIRECTORY_SEPARATOR;
    }

    /**
     * getUploadFolder
     * 
     * @return type
     */
    public static function getUploadFolder()
    {
        return WP_CONTENT_DIR . DIRECTORY_SEPARATOR . 'uploads';
    }

    /**
     * removeTableChart
     * 
     * @param type $description
     * @param type $tagName
     * @param type $text
     * @return type
     */
    public static function removeTableChart($description, $tagName = 'h3', $text = 'Size chart')
    {
        $doc = new \DOMDocument();
        $doc->encoding = 'UTF-8';
        libxml_use_internal_errors(true);
        $doc->recover = true;
        $doc->strictErrorChecking = false;
        $doc->loadHTML(mb_convert_encoding($description, 'HTML-ENTITIES', 'UTF-8'));

        foreach ($doc->getElementsByTagName('table') as $node) {
            while ($node->hasChildNodes()) {
                $node->removeChild($node->firstChild);
            }
            $node->parentNode->removeChild($node);
        }

        $items = $doc->getElementsByTagName($tagName);
        foreach ($items as $key => $node) {
            preg_match('/^' . $text . '/', $node->nodeValue, $matches);
            if (count($matches) > 0) {
                $chartNode = $items->item($key);
                $chartNode->parentNode->removeChild($chartNode);
            }
        }

        $string = '';
        $xpath = new \DOMXPath($doc);
        foreach ($xpath->query('/html/body//text()[normalize-space(.) != ""]', $doc) as $node) {
            if ($node->nodeType == XML_TEXT_NODE) {
                $string .= preg_replace('/\s+/', ' ', str_replace('&#13;', '', $node->nodeValue));
            }
        }

        $clearString = trim($string);
        if (empty($clearString)) {
            return self::getEmptyDescription();
        }

        return self::strip_shortcodes($clearString);
    }

    /**
     * getDocument
     * 
     * @return \DOMDocument
     */
    public static function getDocument()
    {
        $doc = new \DOMDocument();
        libxml_use_internal_errors(true);
        $doc->recover = true;
        $doc->strictErrorChecking = false;
        $doc->load(self::getFilePath());
        return $doc;
    }

    /**
     * addItem
     * 
     * @param type $item
     */
    public static function addItem($item)
    {
        self::$doc->startElement('item');
        self::$doc->writeElement('g:id', $item['itemId']);
        self::$doc->writeElement('g:title', $item['title']);
        self::$doc->writeElement('g:description', $item['description']);
        self::$doc->writeElement('g:link', $item['link']);
        self::$doc->writeElement('g:brand', $item['brand']);
        self::$doc->writeElement('g:price', $item['price']);
        self::$doc->writeElement('g:condition', $item['condition']);
        self::$doc->writeElement('g:availability', $item['availability']);
        self::$doc->writeElement('g:google_product_category', $item['google_product_category']);
        self::addImage($item);
        self::addMaterial($item);
        self::$doc->writeElement('g:item_group_id', $item['id']);
        self::$doc->writeElement('g:age_group', $item['age_group']);
        self::$doc->writeElement('g:gender', $item['gender']);
        if ($item['identifier_exists'] == 0) {
            self::$doc->writeElement('g:mpn', $item['mpn']);
        } else {
            self::$doc->writeElement('g:identifier_exists', 'no');
        }
        self::$doc->endElement();
    }

    /**
     * removeItem
     * 
     * @param type $post_id
     */
    public static function removeItem($post_id)
    {
        $doc = $this->getDocument();
        $xpath = new \DOMXPath($doc);
        foreach ($xpath->query("/rss/channel/item[g:item_group_id='" . $post_id . "']") as $node) {
            while ($node->hasChildNodes()) {
                $node->removeChild($node->firstChild);
            }
            $node->parentNode->removeChild($node);
        }
        $doc->save($this->getFilePath());
    }

    /**
     * createItem
     * 
     * @param type $doc
     * @return type
     */
    public static function createItem($doc)
    {
        $channelNode = $doc->getElementsByTagName('channel')->item(0);
        if (is_null($channelNode)) {
            return false;
        }
        $itemNode = $channelNode->appendChild($doc->createElement('item'));
        return $itemNode;
    }

    /**
     * addImage
     * 
     * @param type $item
     */
    public static function addImage($item)
    {
        $imageColor = false;
        $imageType = false;
        if (isset($item['size']) && !empty($item['size'])) {
            self::$doc->writeElement('g:size', $item['size']);
        }

        if (isset($item['color']) && !empty($item['color'])) {
            self::$doc->writeElement('g:color', $item['color']);
            $imageColor = self::getImage($item['color']);
            if ($imageColor) {
                self::$doc->writeElement('g:image_link', $imageColor);
            }
        }

        if (!$imageColor && isset($item['type']) && !empty($item['type'])) {
            $imageType = self::getImage($item['type']);
            if ($imageType) {
                self::$doc->writeElement('g:image_link', $imageType);
            }
        }

        if (!$imageColor && !$imageType) {
            self::$doc->writeElement('g:image_link', $item['image_link']);
        }
    }

    /**
     * addMaterial
     * 
     * @param type $item
     */
    public static function addMaterial($item)
    {
        if (isset($item['material']) && !empty($item['material'])) {
            self::$doc->writeElement('g:material', $item['material']);
        }
    }

    /**
     * getEmptyDescription
     * 
     * @return type
     */
    public static function getEmptyDescription()
    {
        return '100% brand new and high quality!';
    }

    /**
     * getImages
     * 
     * @return type
     */
    public static function getImages()
    {
        return self::$images;
    }

    /**
     * setImages
     * 
     * @param type $images
     */
    public static function setImages($images)
    {
        if (isset($images['images'])) {
            self::$images = $images;
        }
    }

    /**
     * getImage
     * 
     * @param type $index
     * @return boolean
     */
    public static function getImage($index)
    {
        $images = self::getImages();
        if (isset($images[$index])) {
            return $images[$index];
        }

        return false;
    }

    /**
     * deleteXml
     */
    public static function deleteXml()
    {
        if (!is_dir(self::getFolderDir())) {
            return false;
        }

        $files = array_diff(scandir(self::getFolderDir()), array('.', '..'));
        if (!empty($files)) {
            foreach ($files as $file) {
                unlink(self::getFolderDir() . $file);
            }
        }
    }

    /**
     * strip_shortcodes
     * 
     * @param type $content
     * @return type
     */
    public static function strip_shortcodes($content)
    {
        if (false === strpos($content, '[')) {
            return $content;
	}

        preg_match_all('@\[([^<>&/\[\]\x00-\x20=]++)@', $content, $matches);
        $tagnames = array_intersect(Settings::maybe_unserialize(WP_SHORTCODES), $matches[1]);
        if (empty($tagnames)) {
            return $content;
        }

        foreach ($tagnames as $index => $tag) {
            $content = trim(preg_replace('/\[' . $tag . '\]/', '', $content), '&nbsp;');
        }

        $string = htmlentities($content, null, 'utf-8');
        $content = str_replace("&nbsp;", " ", $string);
        $content = html_entity_decode($content);
        return trim($content);
    }
}